package tasks;

import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;

/**
 * Utilities to read and print graph data from/to standard input/output.
 */
public class Graph {

  /**
   * Read graph data from standard input.
   * @return An adjacency list representation of a directed graph.
   */
  public static List<Edge>[] read() {
    Scanner scanner = new Scanner(System.in);
    int n = scanner.nextInt();
    int m = scanner.nextInt();
    List<Edge>[] adj = (ArrayList<Edge>[]) new ArrayList[n];

    // Create auxiliary list for each vertex
    for (int i = 0; i < n; i++) {
      adj[i] = new ArrayList<>();
    }

    // Add incident edges
    for (int x, y, i = 0; i < m; i++) {
      x = scanner.nextInt();
      y = scanner.nextInt();
      adj[x - 1].add(new Edge(x, y));
    }

    scanner.close();
    return adj;
  }

  /**
   * Print graph data to standard output.
   * The output can be used to "create" visualization of your graph using the GraphViz tool: https://edotor.net/
   * @param adj - An adjacency list representation of a directed graph.
   */
  public static void print(List<Edge>[] adj) {
    String connector = "->";
    StringBuilder sb = new StringBuilder();
    sb.append("digraph {\n");
    sb.append("   rankdir=LR;\n");
    sb.append("   node [shape=circle];\n");

    // add vertices to output
    for (int i = 0; i < adj.length; i++) {
      sb.append("   ").append(i + 1).append("\n");
    }

    // add edges to output
    for (List<Edge> edges : adj) {
      for (Edge e : edges) {
        String edge = String.format("%d %s %s", e.from, connector, e.to);
        sb.append("   ").append(edge).append("\n");
      }
    }

    sb.append("}");
    System.out.println(sb.toString());
  }

  /**
   * Representation of a directed edge.
   */
  public static class Edge {
    int from;
    int to;

    /**
     * Construct a directed edge.
     *
     * @param from - Edge endpoint.
     * @param to - Edge endpoint.
     */
    public Edge(int from, int to) {
      this.from = from;
      this.to = to;
    }
  }
}
